import 'package:flutter/material.dart';
import 'package:chessjs/chessjs.dart' as chessjs;

/// To handle notation. The notation widget subscribe to this model
/// and builds the notation tiles provided this list.
/// Also handles the iteration through the game.
/// It will be used to handle different lines in a position
/// also, as a tree-like structure but,
/// ***FOR NOW THERE IS NO SUPPORT FOR LINES***
class MoveHistory extends ChangeNotifier {
  MoveHistory();

  /// whole move history
  List<String> _history = [];

  /// moves undone when user when back on the game
  List<chessjs.Move> _undoneMoves = [];

  /// current move display on the board
  int _currentIndex = 0;

  /// result of the game
  String? _result;

  List<String> get history => _history;
  List<chessjs.Move> get changedMoves => _undoneMoves;
  int get currentIndex => _currentIndex;
  String? get result => _result;

  /// add a move to the move history
  void addMoveToHistory(String move) {
    _history.add(move);
    _currentIndex = _currentIndex + 1;
    notifyListeners();
  }

  /// add a move to the move history
  void setResult(String result) {
    _result = result;
    notifyListeners();
  }

  /// undo a move completely from history
  void removeMove() {
    if (changedMoves.isNotEmpty) {
      changedMoves.removeAt(0);
      notifyListeners();
    } else if (_history.isNotEmpty) {
      _history.removeLast();
      notifyListeners();
    }
  }

  chessjs.Move? goForwardOneMove() {
    if (changedMoves.isNotEmpty) {
      _currentIndex = _currentIndex + 1;
      final chessjs.Move move = changedMoves.removeLast();
      return move;
    } else
      return null;
  }

  /// decrease current index in history and save the undone move
  void goBackOneMove(chessjs.Move undoneMove) {
    if (_history.isNotEmpty) {
      _currentIndex = _currentIndex - 1;
      changedMoves.add(undoneMove);
    }
  }
}
